# aiLib

# distances
from distance import *

# basic statistics, binning, histogram
from stats import *

# information theory
#from info import *

# distributions additionals
from dists import *

# conversions
from conversion import *


######### ABBREVIATIONS #########

head = lambda lst: lst[0]

tail = lambda lst: lst[1:]

last = lambda lst: lst[-1]

init = lambda lst: lst[:-1]

fst = head

snd = lambda lst: lst[1]



######### BASIC MATH #########

def sign(x):
    """Return -1 if *x* is smaller than zero. If *x* is equal to zero, return zero. Return +1 otherwise."""
    if x < 0: return -1
    if x == 0: return 0
    return 1


sgn = sign


span = lambda lst: max(lst) - min(lst)
"""Return the span of a list."""


prod = lambda lst: reduce(lambda x,y: x*y, lst)
"""Return the product of all list elements."""


def argmin(f,arg):
    """Return the argument for which the value of a function is minimal.

    The function *f* is evaluated at each element of the argument list
    *arg*. The value in *arg* which returns the lowest value of *f*
    is returned.

    :param f:       Function to be evaluated.
    :type f:        (Ord b) => a -> b
    :param arg:     List of argument values.
    :type arg:      a

    >>> l1, l2, f = [4,5,6], [1,2,3], lambda x,y: x**2-y**2
    >>> argmin(lambda (x,y): f(x,y), itertools.product(l1, l2))
    (4, 3)
    
    """
    e = [ (a,f(a)) for a in arg ]
    m = min( range(len(e)), key=lambda i: last(e[i]))
    return head(e[m])

def argmax(f,arg):
    """Return the argument for which the value of a function is maximal.
    
    The function *f* is evaluated at each element of the argument list
    *arg*. The value in *arg* which returns the lowest value of *f*
    is returned.
    
    :param f:       Function to be evaluated.
    :type f:        (Ord b) => a -> b
    :param arg:     List of argument values.
    :type arg:      a
    
    >>> l1, l2, f = [4,5,6], [1,2,3], lambda x,y: x**2-y**2
    >>> argmax(lambda (x,y): f(x,y), itertools.product(l1, l2))
    (6, 1)

    """
    e = [ (a,f(a)) for a in arg ]
    m = max( range(len(e)), key=lambda i: last(e[i]))
    return head(e[m])


def normalize(lst, s=1.0):
    """Return the list, such that the elements sum up to *s*.
    
    :param lst:     A list of numerals.
    :param s:       Normalize scale.
    :type nrm:      Float
    :returns:       Scaled list with sum(*lst*) = *nrm*
    
    >>> normalize([2.0,4.0, 6.0], 3.0)
    [0.5, 1.0, 1.5]
    
    >>> sum(normalize([2.0,4.0, 6.0], 3.0))
    3.0
    
    """
    nrm = sum(lst)
    return map(lambda i: s * float(i)/nrm, lst)


