
from numpy.random import uniform
from ..ailib import normalize

def rouletteWheel(data, weights):
    """Return a sample drawn from a discrete distribution.
    
    :param data:        List of tokens.
    :type data:         [a]
    :param weights:     Weights of tokens.
    :type weights:      [float]
    :returns:           Generator object for samples drawn from *data*
                        according to their weights.

    """
    weights = normalize(weights)
    N = len(weights)
    cw = [sum(weights[:i]) for i in range(1,N+1)]
    # assert( abs(cw[-1] - 1.0) < tol )
    
    while True:
        r = uniform(0.0, 1.0)
        for j in xrange(N):
            if r <= cw[j]:
                yield data[j]
                break

def rouletteWheelN(data, weights, numSamples):
    """Return samples drawn from a discrete distribution.
    
    :param data:        List of tokens.
    :type data:         [a]
    :param weights:     Weights of tokens.
    :type weights:      [float]
    :param numSamples:  Number of samples to draw.
    :type numSamples:   int
    :returns:           *numSamples* samples drawn from *data* according
                        to their weights.

    """
    g = rouletteWheel(data, weights)
    return [g.next() for i in range(numSamples)]


