
from ..ailib import binning

# crossValidation :: [(feature, label)], model, num -> [num]
# alternative loss function (ESLII, p.242)
# return value? (cv_err as in ESLII, p.242 or detailed statistics or set with lowest generalization error)
def crossValidation(data, model, K):
    """
    
    :param data:        Data points.
    :type data:         :term:`STD`
    :param model:       Model to fit the data points to.
    :type model:        :class:`ailib.fitting.Model`
    :param K:           Number of buckets.
    :type K:            int
    :returns:           Cross-validation error.
    
    """
    N = len(data)
    bins = binning(range(N), K)
    err = []
    for i in range(K):
        
        trainData = reduce(lambda x,y: x+y, bins[:i] + bins[(i+1):])
        testData = bins[i]
        
        # fit, measure and report test error
        model.fit(map(lambda i: data[i], trainData))
        testErr  = sum([ model.err(data[s]) for s in testData ])
        err.append( testErr )
        
        # alternatively: report detailed statistics
        #trainErr = sum([ model.err(s) for s in trainData ])
        #err.append( (trainErr, testErr) )
    #return err
    return 1.0/N * sum(err)

