
from random import sample

class Ransac:
    """
    
    """
    def __init__(self, thres, minPts, prob=0.99, maxIter=100):
        """
        :param thres:       Error threshold.
        :type thres:        float
        :param prob:        Minimum probability of finding an outlier-free root
                            set.
        :type prob:         float
        :param maxIter:     Maximum number of iterations.
        :type maxIter:      int
        :param minPts:      Minimum number of points the model requires to be
                            computed.
        :type minPts:       int
        
        """
        self.thres = thres
        self.minProb = prob
        self.maxIter = maxIter
        self.minPts = minPts
    
    def fit(self, data, model):
        """
        
        :param data:        Training data.
        :type data:         :term:`STD`
        :param model:       Model to be fit.
        :type model:        :class:`ailib.fitting.Model`
        :returns:           *model* instance, fitted to the optimal
                            consensus set.
        
        """
        best = []
        numIter, p = 0, 0.0
        while numIter < self.maxIter and p < self.minProb:
            # sample root set
            root = sample(data, self.minPts)
            
            # model fitting
            model.fit(root)
            
            # build and eval. consensus set
            cs = filter(lambda x: model.err(x) < self.thres, data)
            if len(cs) > len(best):
                # plain Ransac
                best = cs
            
            # convergence
            p = 1 - (1 - (float(len(best))/len(data))**(float(self.minPts)))**numIter
            numIter += 1
        
        # fit on largest consensus set
        model.fit(best)
        
        return model


"""

Alternatives:

# sample root set
root = sample(data, self.minPts)

# model fitting
model.fit(root)

# build and eval. consensus set
cs = filter(lambda x: model.err(x) < self.thres, data)

# msac
err = sum( map(model.err, cs) )
err += (len(data) - len(cs)) * (self.thres + const)
if (err < bestErr):
    bestErr = err
    best = cs

# engl. wikipedia
if len(cs) > minModelSize:
    model.fit(cs)
    err = sum([model.err(x) for x in data])
    if err < bestErr:
        best = cs
        bestErr = err
    
# deut. wikipedia
cs = filter(lambda x: model.err(x) < self.thres, data)
if len(cs) > len(best):
    best = cs

# lo-ransac
if len(cs) > len(best):
    # 1. Standard
    # Keep cs
    cs = best

    # 2. Simple
    # ??

    # 3. Iterative
    # Iterate for K=??..1.0
    #   cs_k = model.err(p) < K*thres
    #   model.fit(cs_k)
    # Keep cs_1.0
    
    # 4. Inner Ransac
    # Repeat 10 times:
    #   Sample >=self.minPts from cs
    #   cs' = Test against all data points
    # Keep largest cs'
    
    # 5. Inner Ransac with iteration
    # Repeat 10 times:
    #   Sample >= self.minPts from cs
    #   model.fit(new sample)
    #   Iterate for K=??..1.0
    #     cs_k = model.err(p) < K*thres
    #     model.fit(cs_k)
    #   cs' = cs_1.0
    # Keep largest cs'

"""
